@@ -1,7 +1,6 @@
 import abc
 
 
-#TODO: document me
 class Fitter(object):
     r"""
     Interface that all Fitter objects must implement.
@@ -25,13 +24,29 @@ class Fitter(object):
     def fit(self, image, initial_parameters, gt_shape=None, **kwargs):
         r"""
         Fits the fitter object to an image.
+
+        Parameters
+        -----------
+        image: :class:`menpo.image.masked.MaskedImage`
+            The image to be fitted.
+        initial_parameters: list
+            The initial parameters of the model.
+        gt_shape: :class:`menpo.shape.PointCloud`, optional
+            The original ground truth shape associated to the image.
+
+            Default: None
+
+        Returns
+        -------
+        fitting_result: `menpo.fit.fittingresult`
+            The fitting result.
         """
-        fitting = self._create_fitting(image, initial_parameters,
-                                       gt_shape=gt_shape)
-        return self._fit(fitting, **kwargs)
+        fitting_result = self._create_fitting_result(
+            image, initial_parameters, gt_shape=gt_shape)
+        return self._fit(fitting_result, **kwargs)
 
     @abc.abstractmethod
-    def _create_fitting(self, **kwargs):
+    def _create_fitting_result(self, **kwargs):
         r"""
         Abstract method that defines the fitting result object associated to
         the fitter object.
@@ -47,5 +62,6 @@ class Fitter(object):
 
     def get_parameters(self, shape):
         r"""
+        Abstract method that gets the parameters.
         """
-        return shape
+        pass
