@@ -10,8 +10,7 @@ class ImageFeatures(object):
     Parameters
     ----------
     image :  Any image object
-        The object created by any image class (Image, MaskedImage,
-        SpatialImage, DepthImage).
+        The object created by any image class (Image, MaskedImage).
     """
 
     def __init__(self, image):
@@ -432,12 +431,12 @@ class ImageFeatures(object):
         from menpo.image import MaskedImage, Image
         if isinstance(self._image, MaskedImage):
             # if we have a MaskedImage object
-            feature_image = MaskedImage(feature_pixels)
+            feature_image = MaskedImage(feature_pixels, copy=False)
             # fix mask
             self.transfer_mask(feature_image, window_centres=window_centres)
         else:
             # if we have an Image object
-            feature_image = Image(feature_pixels)
+            feature_image = Image(feature_pixels, copy=False)
         # fix landmarks
         self.transfer_landmarks(feature_image, window_centres=window_centres,
                                 constrain_landmarks=constrain_landmarks)
@@ -470,7 +469,7 @@ class ImageFeatures(object):
         if window_centres is not None:
             if target_image.landmarks.has_landmarks:
                 for l_group in target_image.landmarks:
-                    l = target_image.landmarks[l_group[0]]
+                    l = target_image.landmarks[l_group]
                     # find the vertical and horizontal sampling steps
                     step_vertical = window_centres[0, 0, 0]
                     if window_centres.shape[0] > 1:
