@@ -16,35 +16,44 @@ class MaskedImage(Image):
     Represents an n-dimensional k-channel image, which has a mask.
     Images can be masked in order to identify a region of interest. All
     images implicitly have a mask that is defined as the the entire image.
-    The mask is an instance of
-    :class:`BooleanImage`.
+    The mask is an instance of :map:`BooleanImage`.
 
     Parameters
     ----------
     image_data :  ndarray
         The pixel data for the image, where the last axis represents the
         number of channels.
-    mask : (M, N) ``np.bool`` ndarray or :class:`BooleanImage`, optional
+    mask : (M, N) `np.bool` ndarray or :map:`BooleanImage`, optional
         A binary array representing the mask. Must be the same
         shape as the image. Only one mask is supported for an image (so the
         mask is applied to every channel equally).
 
-        Default: :class:`BooleanImage` covering the whole image
+        Default: :map:`BooleanImage` covering the whole image
 
+    copy: bool, optional
+        If False, the image_data will not be copied on assignment. If a mask is
+        provided, this also won't be copied.
+        In general this should only be used if you know what you are doing.
+
+        Default False
     Raises
     ------
     ValueError
         Mask is not the same shape as the image
-    """
 
-    def __init__(self, image_data, mask=None):
-        super(MaskedImage, self).__init__(image_data)
+    """
+    def __init__(self, image_data, mask=None, copy=True):
+        super(MaskedImage, self).__init__(image_data, copy=copy)
         if mask is not None:
+            # Check if we need to create a BooleanImage or not
             if not isinstance(mask, BooleanImage):
-                mask_image = BooleanImage(mask)
+                # So it's a numpy array.
+                mask_image = BooleanImage(mask, copy=copy)
             else:
+                # It's a BooleanImage object.
+                if copy:
+                    mask = mask.copy()
                 mask_image = mask
-                # have a BooleanImage object that we definitely own
             if mask_image.shape == self.shape:
                 self.mask = mask_image
             else:
@@ -56,17 +65,6 @@ class MaskedImage(Image):
             # no mask provided - make the default.
             self.mask = BooleanImage.blank(self.shape, fill=True)
 
-    # noinspection PyMethodOverriding
-    @classmethod
-    def _init_with_channel(cls, image_data_with_channel, mask):
-        r"""
-        Constructor that always requires the image has a
-        channel on the last axis. Only used by from_vector. By default,
-        just calls the constructor. Subclasses with constructors that don't
-        require channel axes need to overwrite this.
-        """
-        return cls(image_data_with_channel, mask)
-
     @classmethod
     def blank(cls, shape, n_channels=1, fill=0, dtype=np.float, mask=None):
         r"""
@@ -110,8 +108,13 @@ class MaskedImage(Image):
         blank_image : :class:`MaskedImage`
             A new masked image of the requested size.
         """
-        return super(MaskedImage, cls).blank(shape, n_channels=n_channels,
-                                             fill=fill, dtype=dtype, mask=mask)
+        # Ensure that the '+' operator means concatenate tuples
+        shape = tuple(np.ceil(shape).astype(np.int))
+        if fill == 0:
+            pixels = np.zeros(shape + (n_channels,), dtype=dtype)
+        else:
+            pixels = np.ones(shape + (n_channels,), dtype=dtype) * fill
+        return cls(pixels, copy=False, mask=mask)
 
     @property
     def n_true_pixels(self):
@@ -129,18 +132,59 @@ class MaskedImage(Image):
     def n_false_elements(self):
         return self.n_false_pixels * self.n_channels
 
+    @property
+    def indices(self):
+        r"""
+        Return the indices of all true pixels in this image.
+
+        :type: (`n_dims`, `n_true_pixels`) ndarray
+
+        """
+        return self.mask.true_indices
+
     @property
     def masked_pixels(self):
         r"""
-        Get the pixels covered by the ``True`` values in the mask.
+        Get the pixels covered by the `True` values in the mask.
 
-        :type: (``mask.n_true``, ``n_channels``) ndarray
+        :type: (`mask.n_true`, `n_channels`) ndarray
         """
+        if self.mask.all_true:
+            return self.pixels
         return self.pixels[self.mask.mask]
 
-    @masked_pixels.setter
-    def masked_pixels(self, value):
-        self.pixels[self.mask.mask] = value
+    def set_masked_pixels(self, pixels, copy=True):
+        r"""Update the masked pixels only to new values.
+
+        Parameters
+        ----------
+        pixels: ndarray
+            The new pixels to set.
+
+        copy: `bool`, optional
+            If False a copy will be avoided in assignment. This can only happen
+            if the mask is all True - in all other cases it will raise a
+            warning.
+
+        Raises
+        ------
+
+        Warning : If the copy=False flag cannot be honored.
+
+        """
+        if self.mask.all_true:
+            if copy:
+                pixels = pixels.copy()
+            # Our mask is all True, so if they don't want a copy
+            # we can respect their wishes
+            self.pixels = pixels.reshape(self.shape + (self.n_channels,))
+        else:
+            self.pixels[self.mask.mask] = pixels
+            # oh dear, couldn't avoid a copy. Did the user try to?
+            if not copy:
+                raise Warning('The copy flag was NOT honoured. '
+                              'A copy HAS been made. copy can only be avoided'
+                              ' if MaskedImage has an all_true mask.')
 
     def __str__(self):
         return ('{} {}D MaskedImage with {} channels. '
@@ -148,7 +192,27 @@ class MaskedImage(Image):
             self._str_shape, self.n_dims, self.n_channels,
             self.mask.proportion_true))
 
-    def as_vector(self, keep_channels=False):
+    def copy(self):
+        r"""
+        Return a new image with copies of the pixels, landmarks, and masks of
+        this image.
+
+        This is an efficient copy method. If you need to copy all the state on
+        the object, consider deepcopy instead.
+
+        Returns
+        -------
+
+        image: :map:`MaskedImage`
+            A new image with the same pixels, mask and landmarks as this one,
+            just copied.
+
+        """
+        new_image = MaskedImage(self.pixels, mask=self.mask)
+        new_image.landmarks = self.landmarks
+        return new_image
+
+    def _as_vector(self, keep_channels=False):
         r"""
         Convert image to a vectorized form. Note that the only pixels
         returned here are from the masked region on the image.
@@ -160,29 +224,31 @@ class MaskedImage(Image):
             ========== ====================================
             Value      Return shape
             ========== ====================================
-            ``True``   (``mask.n_true``,``n_channels``)
-            ``False``  (``mask.n_true`` x ``n_channels``,)
+            `True`     (`mask.n_true`,`n_channels`)
+            `False`    (`mask.n_true` x `n_channels`,)
             ========== ====================================
 
-            Default: ``False``
+            Default: `False`
 
         Returns
         -------
-        vectorized_image : (shape given by ``keep_channels``) ndarray
+        vectorized_image : (shape given by `keep_channels`) ndarray
             Vectorized image
         """
         if keep_channels:
             return self.masked_pixels.reshape([-1, self.n_channels])
         else:
-            return self.masked_pixels.flatten()
+            return self.masked_pixels.ravel()
 
-    def from_vector(self, flattened, n_channels=None):
+    def from_vector(self, vector, n_channels=None):
         r"""
         Takes a flattened vector and returns a new image formed by reshaping
         the vector to the correct pixels and channels. Note that the only
         region of the image that will be filled is the masked region.
 
-        The ``n_channels`` argument is useful for when we want to add an extra
+        On masked images, the vector is always copied.
+
+        The `n_channels` argument is useful for when we want to add an extra
         channel to an image but maintain the shape. For example, when
         calculating the gradient.
 
@@ -190,10 +256,10 @@ class MaskedImage(Image):
 
         Parameters
         ----------
-        flattened : (``n_pixels``,)
+        vector : (`n_pixels`,)
             A flattened vector of all pixels and channels of an image.
         n_channels : int, optional
-            If given, will assume that flattened is the same
+            If given, will assume that vector is the same
             shape as this image, but with a possibly different number of
             channels
 
@@ -204,21 +270,24 @@ class MaskedImage(Image):
         image : :class:`MaskedImage`
             New image of same shape as this image and the number of
             specified channels.
+
         """
         # This is useful for when we want to add an extra channel to an image
         # but maintain the shape. For example, when calculating the gradient
         n_channels = self.n_channels if n_channels is None else n_channels
         # Creates zeros of size (M x N x ... x n_channels)
-        image_data = np.zeros(self.shape + (n_channels,))
-        pixels_per_channel = flattened.reshape((-1, n_channels))
-        image_data[self.mask.mask] = pixels_per_channel
-        # call the constructor accounting for the fact that some image
-        # classes expect a channel axis and some don't.
-        new_image = type(self)._init_with_channel(image_data, mask=self.mask)
+        if self.mask.all_true:
+            # we can just reshape the array!
+            image_data = vector.reshape((self.shape + (n_channels,)))
+        else:
+            image_data = np.zeros(self.shape + (n_channels,))
+            pixels_per_channel = vector.reshape((-1, n_channels))
+            image_data[self.mask.mask] = pixels_per_channel
+        new_image = MaskedImage(image_data, mask=self.mask)
         new_image.landmarks = self.landmarks
         return new_image
 
-    def from_vector_inplace(self, vector):
+    def from_vector_inplace(self, vector, copy=True):
         r"""
         Takes a flattened vector and updates this image by reshaping
         the vector to the correct pixels and channels. Note that the only
@@ -226,10 +295,23 @@ class MaskedImage(Image):
 
         Parameters
         ----------
-        vector : (``n_pixels``,)
+        vector : (`n_parameters`,)
             A flattened vector of all pixels and channels of an image.
+
+        copy: `bool`, optional
+            If False, the vector will be set as the pixels with no copy made.
+            If True a copy of the vector is taken.
+
+            Default: True
+
+        Raises
+        ------
+
+        Warning : If copy=False cannot be honored.
+
         """
-        self.masked_pixels = vector.reshape((-1, self.n_channels))
+        self.set_masked_pixels(vector.reshape((-1, self.n_channels)),
+                               copy=copy)
 
     def _view(self, figure_id=None, new_figure=False, channels=None,
               masked=True, **kwargs):
@@ -253,7 +335,7 @@ class MaskedImage(Image):
                            pixels_to_view, channels=channels,
                            mask=mask).render(**kwargs)
 
-    def crop(self, min_indices, max_indices,
+    def crop_inplace(self, min_indices, max_indices,
              constrain_to_boundary=True):
         r"""
         Crops this image using the given minimum and maximum indices.
@@ -292,17 +374,17 @@ class MaskedImage(Image):
 
         """
         # crop our image
-        super(MaskedImage, self).crop(
+        super(MaskedImage, self).crop_inplace(
             min_indices, max_indices,
             constrain_to_boundary=constrain_to_boundary)
         # crop our mask
-        self.mask.crop(min_indices, max_indices,
+        self.mask.crop_inplace(min_indices, max_indices,
                        constrain_to_boundary=constrain_to_boundary)
         return self
 
     def crop_to_true_mask(self, boundary=0, constrain_to_boundary=True):
         r"""
-        Crop this image to be bounded just the ``True`` values of it's mask.
+        Crop this image to be bounded just the `True` values of it's mask.
 
         Parameters
         ----------
@@ -313,23 +395,23 @@ class MaskedImage(Image):
             Default: 0
 
         constrain_to_boundary: boolean, optional
-            If ``True`` the crop will be snapped to not go beyond this images
-            boundary. If ``False``, a ImageBoundaryError will be raised if an
+            If `True` the crop will be snapped to not go beyond this images
+            boundary. If `False`, a ImageBoundaryError will be raised if an
             attempt is made to go beyond the edge of the image. Note that is
             only possible if boundary != 0.
 
-            Default: ``True``
+            Default: `True`
 
         Raises
         ------
         ImageBoundaryError
-            Raised if constrain_to_boundary is ``False``, and an attempt is
+            Raised if constrain_to_boundary is `False`, and an attempt is
             made to crop the image in a way that violates the image bounds.
         """
         min_indices, max_indices = self.mask.bounds_true(
             boundary=boundary, constrain_to_bounds=False)
         # no point doing the bounds check twice - let the crop do it only.
-        self.crop(min_indices, max_indices,
+        self.crop_inplace(min_indices, max_indices,
                   constrain_to_boundary=constrain_to_boundary)
 
     def warp_to(self, template_mask, transform, warp_landmarks=False,
@@ -347,22 +429,22 @@ class MaskedImage(Image):
             Defines, for each pixel location on the template, which pixel
             location should be sampled from on this image.
         warp_landmarks : bool, optional
-            If ``True``, warped_image will have the same landmark dictionary
+            If `True`, warped_image will have the same landmark dictionary
             as self, but with each landmark updated to the warped position.
 
-            Default: ``False``
+            Default: `False`
         warp_mask : bool, optional
-            If ``True``, sample the ``image.mask`` at all ``template_image``
+            If `True`, sample the `image.mask` at all `template_image`
             points, setting the returned image mask to the sampled value
-            **within the masked region of ``template_image``**.
+            **within the masked region of `template_image`**.
 
-            Default: ``False``
+            Default: `False`
 
             .. note::
 
-                This is most commonly set ``True`` in combination with an all
-                True ``template_mask``, as this is then a warp of the image
-                and it's full mask. If ``template_mask``
+                This is most commonly set `True` in combination with an all
+                True `template_mask`, as this is then a warp of the image
+                and it's full mask. If `template_mask`
                 has False mask values, only the True region of the mask
                 will be updated, which is rarely the desired behavior,
                 but is possible for completion.
@@ -495,8 +577,8 @@ class MaskedImage(Image):
         Parameters
         ----------
         nullify_values_at_mask_boundaries : bool, optional
-            If ``True`` a one pixel boundary is set to 0 around the edge of
-            the ``True`` mask region. This is useful in situations where
+            If `True` a one pixel boundary is set to 0 around the edge of
+            the `True` mask region. This is useful in situations where
             there is absent data in the image which will cause erroneous
             gradient settings.
 
@@ -504,10 +586,10 @@ class MaskedImage(Image):
 
         Returns
         -------
-        gradient : :class:``MaskedImage``
+        gradient : :class:`MaskedImage`
             The gradient over each axis over each channel. Therefore, the
-            gradient of a 2D, single channel image, will have length ``2``.
-            The length of a 2D, 3-channel image, will have length ``6``.
+            gradient of a 2D, single channel image, will have length `2`.
+            The length of a 2D, 3-channel image, will have length `6`.
         """
         grad_image_pixels = features.gradient(self.pixels)
         grad_image = MaskedImage(grad_image_pixels,
@@ -567,7 +649,7 @@ class MaskedImage(Image):
 
         pwa = PiecewiseAffine(pc, pc)
         try:
-            pwa.apply_inplace(self.mask.all_indices)
+            pwa.apply(self.indices)
         except TriangleContainmentError, e:
             self.mask.from_vector_inplace(~e.points_outside_source_domain)
 
@@ -701,7 +783,7 @@ class MaskedImage(Image):
         image_pyramid = Image.gaussian_pyramid(
             self, n_levels=n_levels, downscale=downscale, sigma=sigma,
             order=order, mode=mode, cval=cval)
-        for j, image in enumerate(image_pyramid):
+        for image in image_pyramid:
             image.mask = self.mask.resize(image.shape)
             yield image
 
