@@ -4,7 +4,7 @@ import numpy as np
 
 class MeshPTSImporter(PTSImporter):
     r"""
-    Implements the :meth:`_build_points` method for meshes. Here, ``x`` is the
+    Implements the :meth:`_build_points` method for meshes. Here, `x` is the
     first axis.
 
     Parameters
@@ -18,19 +18,19 @@ class MeshPTSImporter(PTSImporter):
 
     def _build_points(self, xs, ys):
         """
-        For meshes, ``axis 0 = xs`` and ``axis 1 = ys``. Therefore, return the
+        For meshes, `axis 0 = xs` and `axis 1 = ys`. Therefore, return the
         appropriate points array ordering.
 
         Parameters
         ----------
         xs : (N,) ndarray
-            Row vector of ``x`` coordinates
+            Row vector of `x` coordinates
         ys : (N,) ndarray
-            Row vector of ``y`` coordinates
+            Row vector of `y` coordinates
 
         Returns
         -------
         points : (N, 2) ndarray
-            Array with ``xs`` as the first axis: ``[xs; ys]``
+            Array with `xs` as the first axis: `[xs; ys]`
         """
         return np.hstack([xs, ys])
