@@ -15,10 +15,22 @@ class ColouredTriMesh(TriMesh, Rasterizable):
     ----------
     points : (N, D) ndarray
         The coordinates of the mesh.
-    trilist : (M, 3) ndarray
-        The triangle list for the mesh
-    colours : (N, 3) ndarray
-        The floating point RGB colour per vertex.
+    trilist : (M, 3) ndarray, optional
+        The triangle list for the mesh. If `None`, a Delaunay triangulation
+        will be performed.
+
+        Default: `None`
+    colours : (N, 3) ndarray, optional
+        The floating point RGB colour per vertex. If not given, grey will be
+        assigned to each vertex.
+
+        Default: `None`
+    copy: bool, optional
+        If `False`, the points, trilist and colours will not be copied on
+        assignment.
+        In general this should only be used if you know what you are doing.
+
+        Default: `False`
 
     Raises
     ------
@@ -26,14 +38,39 @@ class ColouredTriMesh(TriMesh, Rasterizable):
         If the number of colour values does not match the number of vertices.
     """
 
-    def __init__(self, points, trilist=None, colours=None):
-        TriMesh.__init__(self, points, trilist=trilist)
+    def __init__(self, points, trilist=None, colours=None, copy=True):
+        TriMesh.__init__(self, points, trilist=trilist, copy=copy)
+        # Handle the settings of colours, either be provided a default grey
+        # set of colours, or copy the given array if necessary
         if colours is None:
             # default to grey
-            colours = np.ones_like(points, dtype=np.float) * 0.5
+            colours_handle = np.ones_like(points, dtype=np.float) * 0.5
+        elif not copy:
+            colours_handle = colours
+        else:
+            colours_handle = colours.copy()
+
         if points.shape[0] != colours.shape[0]:
             raise ValueError('Must provide a colour per-vertex.')
-        self.colours = colours
+        self.colours = colours_handle
+
+    def copy(self):
+        r"""
+        An efficient copy of this ColouredTriMesh.
+
+        Only landmarks and points will be transferred. For a full copy consider
+        using `deepcopy()`.
+
+        Returns
+        -------
+        colouredtrimesh: :map:`ColouredTriMesh`
+            A ColouredTriMesh with the same points, trilist, colours and
+            landmarks as this one.
+        """
+        new_ctm = ColouredTriMesh(self.points, colours=self.colours,
+                                  trilist=self.trilist, copy=True)
+        new_ctm.landmarks = self.landmarks
+        return new_ctm
 
     def _view(self, figure_id=None, new_figure=False, coloured=True, **kwargs):
         r"""
@@ -43,9 +80,9 @@ class ColouredTriMesh(TriMesh, Rasterizable):
         Parameters
         ----------
         coloured : bool, optional
-            If ``True``, render the colours.
+            If `True`, render the colours.
 
-            Default: ``True``
+            Default: `True`
 
         Returns
         -------
@@ -55,7 +92,7 @@ class ColouredTriMesh(TriMesh, Rasterizable):
         Raises
         ------
         DimensionalityError
-            If ``self.n_dims != 3``.
+            If `self.n_dims != 3`.
         """
         if coloured:
             if self.n_dims == 3:
