@@ -2,26 +2,30 @@ import abc
 import copy
 import numpy as np
 
+from menpo.base import DX, DP
 from .base import Homogeneous, HomogFamilyAlignment
 
 
-class Affine(Homogeneous):
-    r"""
-    The base class for all n-dimensional affine transformations. Provides
+class Affine(Homogeneous, DP, DX):
+    r"""Base class for all n-dimensional affine transformations. Provides
     methods to break the transform down into it's constituent
     scale/rotation/translation, to view the homogeneous matrix equivalent,
     and to chain this transform with other affine transformations.
 
     Parameters
     ----------
-    h_matrix : (n_dims + 1, n_dims + 1) ndarray
+    h_matrix : ``(n_dims + 1, n_dims + 1)`` `ndarray`
         The homogeneous matrix of the affine transformation.
+
+    copy : `bool`, optional
+        If ``False`` avoid copying ``h_matrix`` for performance.
+
+    skip_checks : `bool`, optional
+        If ``True`` avoid sanity checks on ``h_matrix`` for performance.
     """
-    def __init__(self, h_matrix):
-        Homogeneous.__init__(self, h_matrix)
-        # Affine is a little more constrained (only 2D or 3D supported)
-        # so run our verification
-        Affine.set_h_matrix(self, h_matrix)
+    def __init__(self, h_matrix, copy=True, skip_checks=False):
+        Homogeneous.__init__(self, h_matrix, copy=copy,
+                             skip_checks=skip_checks)
 
     @classmethod
     def identity(cls, n_dims):
@@ -31,58 +35,66 @@ class Affine(Homogeneous):
     def h_matrix(self):
         return self._h_matrix
 
-    def set_h_matrix(self, value):
-        r"""
-        Updates the h_matrix, performing sanity checks.
+    def _set_h_matrix(self, value, copy=True, skip_checks=False):
+        r"""Updates the h_matrix, performing sanity checks.
 
-        The Affine h_matrix is limited in what values are allowed. Account
-        for them here.
+        Parameters
+        ----------
+        value : ndarray
+            The new homogeneous matrix to set
+
+        copy : `bool`, optional
+            If False do not copy the h_matrix. Useful for performance.
+
+        skip_checks : `bool`, optional
+            If True skip sanity checks on the matrix. Useful for performance.
         """
-        shape = value.shape
-        if len(shape) != 2 and shape[0] != shape[1]:
-            raise ValueError("You need to provide a square homogeneous matrix")
-        if self.h_matrix is not None:
-            # already have a matrix set! The update better be the same size
-            if self.n_dims != shape[0] - 1:
-                raise ValueError("Trying to update the homogeneous "
-                                 "matrix to a different dimension")
-        if shape[0] - 1 not in [2, 3]:
-            raise ValueError("Affine Transforms can only be 2D or 3D")
-        if not (np.allclose(value[-1, :-1], 0) and
-                np.allclose(value[-1, -1], 1)):
-            raise ValueError("Bottom row must be [0 0 0 1] or [0, 0, 1]")
-        self._h_matrix = value.copy()
+        if not skip_checks:
+            shape = value.shape
+            if len(shape) != 2 or shape[0] != shape[1]:
+                raise ValueError("You need to provide a square homogeneous "
+                                 "matrix")
+            if self.h_matrix is not None:
+                # already have a matrix set! The update better be the same size
+                if self.n_dims != shape[0] - 1:
+                    raise ValueError("Trying to update the homogeneous "
+                                     "matrix to a different dimension")
+            if shape[0] - 1 not in [2, 3]:
+                raise ValueError("Affine Transforms can only be 2D or 3D")
+            if not (np.allclose(value[-1, :-1], 0) and
+                    np.allclose(value[-1, -1], 1)):
+                raise ValueError("Bottom row must be [0 0 0 1] or [0, 0, 1]")
+        if copy:
+            value = value.copy()
+        self._h_matrix = value
 
     @property
     def linear_component(self):
-        r"""
-        Returns just the linear transform component of this affine
-        transform.
+        r"""The linear component of this affine transform.
 
-        :type: (D, D) ndarray
+        :type: ``(n_dims, n_dims)`` `ndarray`
         """
         return self.h_matrix[:-1, :-1]
 
     @property
     def translation_component(self):
-        r"""
-        Returns just the translation component.
+        r"""The translation component of this affine transform.
 
-        :type: (D,) ndarray
+        :type: ``(n_dims,)`` `ndarray`
         """
         return self.h_matrix[:-1, -1]
 
     def decompose(self):
-        r"""
-        Uses an SVD to decompose this transform into discrete Affine
-        Transforms.
+        r"""Decompose this transform into discrete Affine Transforms.
+
+        Useful for understanding the effect of a complex composite transform.
 
         Returns
         -------
-        transforms : list of :map:`DiscreteAffine` that
-            Equivalent to this affine transform, such that:
+        transforms : list of :map:`DiscreteAffine`
+            Equivalent to this affine transform, such that::
 
-            ``reduce(lambda x,y: x.chain(y), self.decompose()) == self``
+                reduce(lambda x,y: x.chain(y), self.decompose()) == self
         """
         from .rotation import Rotation
         from .translation import Translation
@@ -94,9 +106,6 @@ class Affine(Homogeneous):
         translation = Translation(self.translation_component)
         return [rotation_1, scale, rotation_2, translation]
 
-    def __eq__(self, other):
-        return np.allclose(self.h_matrix, other.h_matrix)
-
     def _transform_str(self):
         r"""
         A string representation explaining what this affine transform does.
@@ -131,7 +140,7 @@ class Affine(Homogeneous):
     @property
     def n_parameters(self):
         r"""
-        ``n_dims * (n_dims + 1)`` parameters - every element of the matrix bar
+        `n_dims * (n_dims + 1)` parameters - every element of the matrix bar
         the homogeneous part.
 
         :type: int
@@ -151,7 +160,7 @@ class Affine(Homogeneous):
         """
         return self.n_dims * (self.n_dims + 1)
 
-    def as_vector(self):
+    def _as_vector(self):
         r"""
         Return the parameters of the transform as a 1D array. These parameters
         are parametrised as deltas from the identity warp. This does not
@@ -167,19 +176,19 @@ class Affine(Homogeneous):
         p2        Affine parameter
         p3        Affine parameter
         p4        Affine parameter
-        p5        Translation in ``x``
-        p6        Translation in ``y``
+        p5        Translation in `x`
+        p6        Translation in `y`
         ========= ===========================================
 
         3D and higher transformations follow a similar format to the 2D case.
 
         Returns
         -------
-        params : (P,) ndarray
-            The values that paramaterise the transform.
+        params : ``(n_parameters,)`` `ndarray`
+            The values that parametrise the transform.
         """
         params = self.h_matrix - np.eye(self.n_dims + 1)
-        return params[:self.n_dims, :].flatten(order='F')
+        return params[:self.n_dims, :].ravel(order='F')
 
     def from_vector_inplace(self, p):
         r"""
@@ -196,19 +205,20 @@ class Affine(Homogeneous):
         else:
             ValueError("Only 2D (6 parameters) or 3D (12 parameters) "
                        "homogeneous matrices are supported.")
-        self.set_h_matrix(h_matrix)
+        self.set_h_matrix(h_matrix, copy=False, skip_checks=True)
 
     @property
     def composes_inplace_with(self):
         return Affine
 
     def _build_pseudoinverse(self):
-        return Affine(np.linalg.inv(self.h_matrix))
+        # Skip the checks as we know inverse of a homogeneous is a homogeneous
+        return Affine(np.linalg.inv(self.h_matrix), copy=False,
+                      skip_checks=True)
 
-    def jacobian(self, points):
-        r"""
-        Computes the Jacobian of the transform w.r.t the parameters. This is
-        constant for affine transforms.
+    def d_dp(self, points):
+        r"""The first order derivative of this Affine transform wrt parameter
+        changes evaluated at points.
 
         The Jacobian generated (for 2D) is of the form::
 
@@ -223,14 +233,15 @@ class Affine(Homogeneous):
 
         Parameters
         ----------
-        points : (N, D) ndarray
+        points : (n_points, n_dims) ndarray
             The set of points to calculate the jacobian for.
 
+
         Returns
         -------
-        dW_dp : (N, P, D) ndarray
-            A (``n_points``, ``n_params``, ``n_dims``) array representing
-            the Jacobian of the transform.
+        (n_points, n_params, n_dims) ndarray
+            The jacobian wrt parametrization
+
         """
         n_points, points_n_dim = points.shape
         if points_n_dim != self.n_dims:
@@ -260,10 +271,10 @@ class Affine(Homogeneous):
         jac[:, full_mask] = 1
         return jac
 
-    def jacobian_points(self, points):
+    def d_dx(self, points):
         r"""
-        Computes the Jacobian of the transform wrt the points to which
-        the transform is applied to. This is constant for affine transforms.
+        The first order derivative of this Affine transform wrt spatial changes
+        evaluated at points.
 
         The Jacobian for a given point (for 2D) is of the form::
 
@@ -276,17 +287,32 @@ class Affine(Homogeneous):
             W(x;p) = [1 + a   -b      tx] [x]
                      [b       1 + a   ty] [y]
                                           [1]
+        Hence it is simply the linear component of the transform.
+
+        Parameters
+        ----------
+
+        points: ndarray shape (n_points, n_dims)
+            The spatial points at which the derivative should be evaluated.
 
         Returns
         -------
-        dW/dx: dW/dx: (N, D, D) ndarray
-            The Jacobian of the transform wrt the points to which the
-            transform is applied to.
+
+        d_dx: (1, n_dims, n_dims) ndarray
+            The jacobian wrt spatial changes.
+
+            d_dx[0, j, k] is the scalar differential change that the
+            j'th dimension of the i'th point experiences due to a first order
+            change in the k'th dimension.
+
+            Note that because the jacobian is constant across space the first
+            axis is length 1 to allow for broadcasting.
+
         """
         return self.linear_component[None, ...]
 
 
-class AlignmentAffine(Affine, HomogFamilyAlignment):
+class AlignmentAffine(HomogFamilyAlignment, Affine):
     r"""
     Constructs an Affine by finding the optimal affine transform to align
     source to target.
@@ -325,7 +351,7 @@ class AlignmentAffine(Affine, HomogFamilyAlignment):
         HomogFamilyAlignment.__init__(self, source, target)
         # now, the Affine
         optimal_h = self._build_alignment_h_matrix(source, target)
-        Affine.__init__(self, optimal_h)
+        Affine.__init__(self, optimal_h, copy=False, skip_checks=True)
 
     @staticmethod
     def _build_alignment_h_matrix(source, target):
@@ -336,29 +362,68 @@ class AlignmentAffine(Affine, HomogFamilyAlignment):
         b = target.h_points
         return np.linalg.solve(np.dot(a, a.T), np.dot(a, b.T)).T
 
-    def set_h_matrix(self, value):
+    def set_h_matrix(self, value, copy=True, skip_checks=False):
         r"""
-        Upon updating the h_matrix we must resync the target.
-        """
-        Affine.set_h_matrix(self, value)
-        # now update the state
-        self._sync_target_from_state()
+        Updates ``h_matrix``, optionally performing sanity checks.
 
-    def from_vector_inplace(self, p):
-        r"""
-        Updates this Affine in-place from the new parameters. See
-        from_vector for details of the parameter format.
+        .. note::
+
+            Updating the ``h_matrix`` on an :map:`AlignmentAffine`
+            triggers a sync of the target.
+
+        Note that it won't always be possible to manually specify the
+        ``h_matrix`` through this method, specifically if changing the
+        ``h_matrix`` could change the nature of the transform. See
+        :attr:`h_matrix_is_mutable` for how you can discover if the
+        ``h_matrix`` is allowed to be set for a given class.
+
+        Parameters
+        ----------
+        value : ndarray
+            The new homogeneous matrix to set
+        copy : `bool`, optional
+            If False do not copy the h_matrix. Useful for performance.
+        skip_checks : `bool`, optional
+            If True skip checking. Useful for performance.
+
+        Raises
+        ------
+        NotImplementedError
+            If :attr:`h_matrix_is_mutable` returns ``False``.
+
+
+        Parameters
+        ----------
+
+        value : ndarray
+            The new homogeneous matrix to set
+
+        copy : bool, optional
+            If False do not copy the h_matrix. Useful for performance.
+
+        skip_checks : bool, optional
+            If True skip verification for performance.
         """
-        Affine.from_vector_inplace(self, p)
+        Affine.set_h_matrix(self, value, copy=copy, skip_checks=skip_checks)
+        # now update the state
         self._sync_target_from_state()
 
     def _sync_state_from_target(self):
         optimal_h = self._build_alignment_h_matrix(self.source, self.target)
         # Use the pure Affine setter (so we don't get syncing)
-        Affine.set_h_matrix(self, optimal_h)
+        # We know the resulting affine is correct so skip the checks
+        Affine.set_h_matrix(self, optimal_h, copy=False, skip_checks=True)
 
-    def copy_without_alignment(self):
-        return Affine(self.h_matrix.copy())
+    def as_non_alignment(self):
+        r"""Returns a copy of this affine without it's alignment nature.
+
+        Returns
+        -------
+        transform : :map:`Affine`
+            A version of this affine with the same transform behavior but
+            without the alignment logic.
+        """
+        return Affine(self.h_matrix, skip_checks=True)
 
 
 class DiscreteAffine(object):
@@ -366,7 +431,7 @@ class DiscreteAffine(object):
     A discrete Affine transform operation (such as a :meth:`Scale`,
     :class:`Translation` or :meth:`Rotation`). Has to be able to invertable.
     Make sure you inherit from :class:`DiscreteAffine` first,
-    for optimal ``decompose()`` behavior.
+    for optimal `decompose()` behavior.
     """
 
     __metaclass__ = abc.ABCMeta
@@ -379,6 +444,6 @@ class DiscreteAffine(object):
         Returns
         -------
         transform : :class:`DiscreteAffine`
-            Deep copy of ``self``.
+            Deep copy of `self`.
         """
-        return [copy.deepcopy(self)]
+        return [self.copy()]
