@@ -1,6 +1,7 @@
 import abc
 import numpy as np
 
+from menpo.base import DX, DL
 from menpo.transform import Affine
 from menpo.transform.base import Alignment, Invertible, Transform
 from .fastpwa import CLookupPWA
@@ -22,7 +23,7 @@ class TriangleContainmentError(Exception):
 
 
 # Note we inherit from Alignment first to get it's n_dims behavior
-class AbstractPWA(Alignment, Transform, Invertible):
+class AbstractPWA(Alignment, Transform, Invertible, DX, DL):
     r"""
     A piecewise affine transformation. This is composed of a number of
     triangles defined be a set of source and target vertices. These vertices
@@ -50,7 +51,7 @@ class AbstractPWA(Alignment, Transform, Invertible):
 
     TriangleContainmentError
         All points to apply must be contained in a source triangle. Check
-        ``error.points_outside_source_domain`` to handle this case.
+        `error.points_outside_source_domain` to handle this case.
     """
     def __init__(self, source, target):
         from menpo.shape import TriMesh  # to avoid circular import
@@ -75,7 +76,7 @@ class AbstractPWA(Alignment, Transform, Invertible):
         r"""
         The triangle list.
 
-        :type: (``n_tris``, 3) ndarray
+        :type: (`n_tris`, 3) ndarray
         """
         return self.source.trilist
 
@@ -100,7 +101,7 @@ class AbstractPWA(Alignment, Transform, Invertible):
         Returns
         -------
         tri_index : (L,) ndarray
-            triangle index for each of the ``points``, assigning each
+            triangle index for each of the `points`, assigning each
             point to it's containing triangle.
         alpha : (L,) ndarray
             Alpha for containing triangle of each point.
@@ -110,8 +111,8 @@ class AbstractPWA(Alignment, Transform, Invertible):
         Raises
         ------
         TriangleContainmentError
-        All ``points`` must be contained in a source triangle. Check
-        ``error.points_outside_source_domain`` to handle this case.
+        All `points` must be contained in a source triangle. Check
+        `error.points_outside_source_domain` to handle this case.
         """
         pass
 
@@ -125,39 +126,53 @@ class AbstractPWA(Alignment, Transform, Invertible):
         new_target = PointCloud(self.source.points)
         return type(self)(new_source, new_target)
 
-    def jacobian_points(self, points):
+    def d_dx(self, points):
         """
-        Calculates the Jacobian of the PWA warp with respect to the the points
-        to which the warp is applied to. Expected to return a
-        ``(n_points, n_dims, n_dims)`` shaped array, so the result is tiled
-        as necessary.
+        Calculates the first order spatial derivative of PWA at points.
 
-        The derivative of a piecewise affine warp with respect to the points
-        is simply the identity matrix for every point in the warp.
+        The nature of this derivative is complicated by the piecewise nature
+        of this transform. For points close to the source points of the
+        transform the derivative is ill-defined. In these cases, an identity
+        jacobian is returned.
+
+        In all other cases the jacobian is equal to the containing triangle's
+        d_dx.
+
+        WARNING - presently the above behavior is only valid at the source
+        points.
 
         Returns
         -------
-        dW/dx: (N, D, D) ndarray
-            The Jacobian of the transform with respect to the points to which
-            the transform is applied to.
-        """
-        return np.tile(np.eye(2, 2), [self.n_points, 1, 1])
+        d_dx: (n_points, n_dims, n_dims) ndarray
+            The first order spatial derivative of this transform
 
-    def weight_points(self, points):
+        Raises
+        ------
+        TriangleContainmentError:
+            If any point is outside any triangle of this PWA.
+
+
+        """
+        # TODO check for position and return true d_dx (see docstring)
+        # for the time being we assume the points are on the source landmarks
+        return np.eye(2, 2)[None, ...]
+
+    def d_dl(self, points):
         """
         Returns the jacobian of the warp at each point given in relation to the
         source points.
 
         Parameters
         ----------
-        points : (K, 2) ndarray
+        points : (n_points, 2) ndarray
             The points to calculate the Jacobian for.
 
         Returns
         -------
-        jacobian : (K, ``n_points``, 2) ndarray
-            The Jacobian for each of the ``K`` given points over each point in
+        d_dl : (n_points, n_centres, 2) ndarray
+            The Jacobian for each of the given points over each point in
             the source points.
+
         """
         tri_index, alpha_i, beta_i = self.index_alpha_beta(points)
         # for the jacobian we only need
@@ -201,7 +216,7 @@ class DiscreteAffinePWA(AbstractPWA):
     r"""
     A piecewise affine transformation.
 
-    Builds ``Affine`` objects for each triangle. apply involves
+    Builds `Affine` objects for each triangle. apply involves
     finding the containing triangle for each input point, and then applying
     the appropriate Affine Transform.
 
@@ -227,7 +242,7 @@ class DiscreteAffinePWA(AbstractPWA):
 
     TriangleContainmentError
         All points to apply_inplace must be contained in a source triangle. Check
-        ``error.points_outside_source_domain`` to handle this case.
+        `error.points_outside_source_domain` to handle this case.
     """
     def __init__(self, source, target):
         AbstractPWA.__init__(self, source, target)
@@ -300,7 +315,7 @@ class DiscreteAffinePWA(AbstractPWA):
         Returns
         -------
         tri_index : (L,) ndarray
-            triangle index for each of the ``points``, assigning each
+            triangle index for each of the `points`, assigning each
             point to it's containing triangle.
         alpha : (L,) ndarray
             Alpha for containing triangle of each point.
@@ -311,8 +326,8 @@ class DiscreteAffinePWA(AbstractPWA):
         Raises
         ------
         TriangleContainmentError
-        All ``points`` must be contained in a source triangle. Check
-        ``error.points_outside_source_domain`` to handle this case.
+        All `points` must be contained in a source triangle. Check
+        `error.points_outside_source_domain` to handle this case.
         """
         alpha, beta = self.alpha_beta(points)
         each_point = np.arange(points.shape[0])
@@ -332,11 +347,11 @@ class DiscreteAffinePWA(AbstractPWA):
 
         Returns
         --------
-        alpha : (K, ``n_tris``)
+        alpha : (K, `n_tris`)
             The alpha for each point and triangle. Alpha can be interpreted
             as the contribution of the ij vector to the position of the
             point in question.
-        beta : (K, ``n_tris``)
+        beta : (K, `n_tris`)
             The beta for each point and triangle. Beta can be interpreted as
              the contribution of the ik vector to the position of the point
              in question.
@@ -356,30 +371,30 @@ class DiscreteAffinePWA(AbstractPWA):
     @staticmethod
     def _containment_from_alpha_beta(alpha, beta):
         r"""
-        Check ``alpha`` and ``beta`` are within a triangle (``alpha >= 0``,
-        ``beta >= 0``, ``alpha + beta <= 1``). Returns the indices of the
-        triangles that are ``alpha`` and ``beta`` are in. If any of the
+        Check `alpha` and `beta` are within a triangle (`alpha >= 0`,
+        `beta >= 0`, `alpha + beta <= 1`). Returns the indices of the
+        triangles that are `alpha` and `beta` are in. If any of the
         points are not contained in a triangle,
         raises a TriangleContainmentError.
 
         Parameters
         ----------
-        alpha: (K, ``n_tris``) ndarray
+        alpha: (K, `n_tris`) ndarray
             Alpha for each point and triangle being tested.
-        beta: (K, ``n_tris``) ndarray
+        beta: (K, `n_tris`) ndarray
             Beta for each point and triangle being tested.
 
         Returns
         -------
         tri_index : (L,) ndarray
-            triangle index for each ``points``, assigning each
+            triangle index for each `points`, assigning each
             point in a triangle to the triangle index.
 
         Raises
         ------
         TriangleContainmentError
-        All ``points`` must be contained in a source triangle. Check
-        ``error.points_outside_source_domain`` to handle this case.
+        All `points` must be contained in a source triangle. Check
+        `error.points_outside_source_domain` to handle this case.
         """
         # (K, n_tris), boolean for whether a given triangle contains a given
         #  point
@@ -458,7 +473,7 @@ class CachedPWA(AbstractPWA):
 
     TriangleContainmentError
         All points to apply must be contained in a source triangle. Check
-        ``error.points_outside_source_domain`` to handle this case.
+        `error.points_outside_source_domain` to handle this case.
     """
     def __init__(self, source, target):
         super(CachedPWA, self).__init__(source, target)
@@ -518,3 +533,4 @@ class CachedPWA(AbstractPWA):
         return (self.ti[tri_index] +
                 alpha[:, None] * self.tij[tri_index] +
                 beta[:, None] * self.tik[tri_index])
+
